km.cutoffs <-
function(dataSet, modName="ME4", timeVar, depVar, modelType="log-rank",
                       ## modelType can equal "log-rank" or "coxph"
                       covars=NULL){
    ## This function was written by Meghan on 2020-4-28, to find cutoffs in eigenprotein values
    ## that optimize the p-value for a logrank test between two groups.
    print(paste0("Finding best cutoff for ", modName, " module with ", depVar, " outcome..."))
    ## Initialize results
    result <- list()
    col <- dataSet[, modName]
    dataSet$zcol1 <- (col - mean(col))/sd(col)
    protVec <- dataSet$zcol1
    best <- c(pVal=1, thresh=0)
    pVals <- c()
    threshs <- c()
    moduleNum <- grep(colnames(dataSet), pattern="ME")
    moduleNum <- moduleNum-1 ## Excluding ME0
    ## Test each threshold
    for(thresh in (1:nrow(dataSet))/nrow(dataSet)){
        dataSet$eigcat <- NA
        dataSet$eigcat[which(dataSet$zcol1 <   quantile(protVec, thresh))]  <- "low"
        dataSet$eigcat[which(dataSet$zcol1 >=  quantile(protVec, thresh))]  <- "high"
        ##
        form0 <- paste0("Surv(", timeVar, ", ", depVar, ") ~ eigcat")
        if(modelType=="log-rank"){
            form1 <- as.formula(form0)
            survdiff1  <- try(survdiff(form1, data=dataSet), silent=TRUE)
            if(inherits(survdiff1, "try-error")) ## e.g., if timeVar is NA for all lows.
               next
            pVal <- pchisq(survdiff1$chisq, length(survdiff1$n)-1,
                           lower.tail=FALSE, log.p=FALSE)
        }
        if(modelType=="coxph"){            
            form2 <- form0
            if(!is.null(covars)){ ## Adding covars:
                form2 <- paste(form0, " + ", paste(covars, collapse = " + "))
            }
            form2 <- as.formula(form2) 
            coxph1 <- try(coxph(form2, data = dataSet), silent=TRUE)
            if(inherits(coxph1, "try-error")) ## e.g., if timeVar is NA for all lows.
                next
            summCoxph1 <- summary(coxph1)
            pVal <- summCoxph1$coefficients["eigcatlow","Pr(>|z|)"]
        }

        pVal <- min(pVal*moduleNum, 1)  
        pVals <- c(pVals, pVal)
        threshs <- c(threshs, thresh)
        eventTab <- table(dataSet$eigcat, dataSet[,depVar])
        eventLow  <- eventTab["low", "1"]
        eventHigh <- eventTab["high","1"]
        event10pct <- (eventLow + eventHigh)/10
        
        if(pVal< best["pVal"] & eventHigh > event10pct){
            best <- c(pVal=pVal, thresh=thresh)
            bestThresh <- thresh
            bestTab <- eventTab
        }
    }
    result[["pVals"]] <- pVals
    result[["thresholds"]] <- threshs
    result[["best"]] <- best
    result[["bestThresh"]] <- bestThresh
    result[["eventTable"]] <- bestTab
    return(result)    
}
